// src/components/h-cesium-viewer/useHCesiumViewerClsSatellite.ts

import type { MaybeRefOrGetter } from 'vue';

import type { HCesiumManager } from './managers/HCesiumManager'; // 导入新的 Viewer Manager
import type { I卫星 } from './managers/HCesiumManager.types'; // 类型定义保持不变

import { SatelliteCalculator } from './calculators/SatelliteCalculator'; // 导入计算器
import { SatelliteManager } from './managers/HCesiumManager.卫星'; // 导入 Satellite Manager

/**
 * 管理 Cesium Viewer 中的卫星实体，根据选中的 ID 列表进行同步。
 * @param hCesiumViewerManager - HCesiumViewerManager 实例或其 getter。
 * @param satelliteList - 包含所有可用卫星选项的数组或 getter。
 * @param selectedSatelliteIds - 包含当前选中卫星 ID 的 Set 或 getter。
 */
export function useHCesiumViewerClsSatellite(
  hCesiumViewerManager: MaybeRefOrGetter<HCesiumManager | null>, // 更新参数类型和名称
  satelliteList: MaybeRefOrGetter<Array<I卫星> | undefined>,
  selectedSatelliteIds: MaybeRefOrGetter<Set<string> | undefined>,
) {
  // SatelliteManager 和 Calculator 实例引用
  const satelliteManager = ref<null | SatelliteManager>(null); // 正确的类型顺序
  // Calculator 通常是无状态的，可以在 setup 作用域创建一次
  const satelliteCalculator = new SatelliteCalculator();

  // 创建一个从 ID 到卫星选项的映射，方便查找
  const satelliteMap = computed(() => {
    const map = new Map<string, I卫星>();
    const list = toValue(satelliteList) ?? [];
    for (const satellite of list) {
      map.set(satellite.id, satellite);
    }
    return map;
  });

  // 使用 watch 显式监听依赖项
  watch(
    [hCesiumViewerManager, satelliteList, selectedSatelliteIds], // 监听这些源的变化
    () => {
      // 回调函数
      const viewerManagerInstance = toValue(hCesiumViewerManager);
      // 检查 Viewer Manager 和内部 Viewer 实例是否存在
      if (!viewerManagerInstance || !viewerManagerInstance.getViewer()) {
        // 如果 viewer manager 或 viewer 实例尚未初始化或已销毁，则清理旧 manager 并返回
        satelliteManager.value?.destroy();
        satelliteManager.value = null;
        return;
      }

      // 确保 SatelliteManager 实例存在且与当前的 ViewerManager 关联
      if (!satelliteManager.value) {
        // 创建 SatelliteManager 时传入 ViewerManager 和 Calculator
        satelliteManager.value = new SatelliteManager(viewerManagerInstance, satelliteCalculator);
      }

      const manager = satelliteManager.value; // 使用 manager 实例

      const selectedIdsSet = toValue(selectedSatelliteIds) ?? new Set<string>(); // 直接获取 Set，如果为 undefined 则创建空 Set
      const currentEntityIds = new Set(manager.getCurrentSatelliteEntities().keys()); // 从 manager 获取当前实体 ID

      // 1. 移除不再选中的卫星
      for (const entityId of currentEntityIds) {
        if (!selectedIdsSet.has(entityId)) {
          manager.removeSatellite(entityId); // 使用 manager 的移除方法
        }
      }

      // 2. 添加新增的选中卫星
      for (const selectedId of selectedIdsSet) {
        if (!currentEntityIds.has(selectedId)) {
          const satelliteToAdd = satelliteMap.value.get(selectedId);
          if (satelliteToAdd) {
            manager.addOrUpdateSatellite(satelliteToAdd); // 使用 manager 的添加/更新方法
          } else {
            // 如果在 satelliteList 中找不到对应的卫星信息，发出警告
            console.warn(`无法找到 ID 为 "${selectedId}" 的卫星信息，无法添加到地图。`);
          }
        }
      }

    },
    { immediate: true, deep: false }, // 立即执行一次，非深度监听
  ); // watch 结束

  // 组件卸载时确保最终清理
  onBeforeUnmount(() => {
    satelliteManager.value?.destroy();
    satelliteManager.value = null; // 明确置空
  });

  // 返回 satelliteMap 可能在某些场景下有用
  return {
    satelliteMap,
  };
}
